<?php

namespace Drupal\cryptolog;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines Cryptolog settings form.
 */
class ConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    TypedConfigManagerInterface $typedConfigManager,
    protected CryptologMiddlewareInterface $cryptologMiddleware,
    protected DateFormatterInterface $dateFormatter,
  ) {
    parent::__construct($configFactory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('cryptolog.middleware'),
      $container->get('date.formatter'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cryptolog_settings';
  }

  /**
   * {@inheritdoc}
   *
   * @return string[]
   *   The editable config names.
   */
  protected function getEditableConfigNames() {
    return [
      'cryptolog.settings',
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @param mixed[] $form
   *   The settings form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return mixed[]
   *   The settings form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('cryptolog.settings');
    $form['single_webhead'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('This site has a single webhead; store salt solely in APCu cache'),
      '#default_value' => $config->get('single_webhead'),
      '#description' => $this->t("If enabled and APCu is available, Cryptolog will store the salt solely in the APCu cache. Otherwise, Cryptolog will share the salt between multiple webheads, storing it in the database unless <code>\$settings['cache']['default']</code> is configured to use Memcache, Redis, etc."),
    ];
    $floodConfig = $this->config('user.flood');
    $form['ttl'] = [
      '#type' => 'number',
      '#title' => $this->t('Salt time to live (seconds)'),
      '#default_value' => $config->get('ttl'),
      '#min' => $floodConfig->get('uid_only') ? $floodConfig->get('ip_window') : $floodConfig->get('user_window'),
      '#description' => $this->t('When this window expires, the salt will be regenerated and each client IP address will map to a new Cryptolog ephemeral identifier. The default is %ttl seconds (@formatted).', [
        '%ttl' => $this->cryptologMiddleware::TTL,
        '@formatted' => $this->dateFormatter->formatInterval($this->cryptologMiddleware::TTL),
      ]),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * @param mixed[] $form
   *   The settings form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('cryptolog.settings')
      ->set('single_webhead', $form_state->getValue('single_webhead'))
      ->set('ttl', $form_state->getValue('ttl'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
